//=---------------------------------------------------------------------=
//
// $Id$ $Name$
//
// The contents of this file are subject to the AAF SDK Public Source
// License Agreement Version 2.0 (the "License"); You may not use this
// file except in compliance with the License.  The License is available
// in AAFSDKPSL.TXT, or you may obtain a copy of the License from the
// Advanced Media Workflow Association, Inc., or its successor.
//
// Software distributed under the License is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See
// the License for the specific language governing rights and limitations
// under the License.  Refer to Section 3.3 of the License for proper use
// of this Exhibit.
//
// WARNING:  Please contact the Advanced Media Workflow Association,
// Inc., for more information about any additional licenses to
// intellectual property covering the AAF Standard that may be required
// to create and distribute AAF compliant products.
// (http://www.amwa.tv/policies).
//
// Copyright Notices:
// The Original Code of this file is Copyright 1998-2009, licensor of the
// Advanced Media Workflow Association.  All rights reserved.
//
// The Initial Developer of the Original Code of this file and the
// licensor of the Advanced Media Workflow Association is
// Avid Technology.
// All rights reserved.
//
// Portions created by British Broadcasting Corporation are
// Copyright 2004, British Broadcasting Corporation.  All rights reserved.
//
//=---------------------------------------------------------------------=


#include "AAF.h"

#include <iostream>
using namespace std;
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "AAFStoredObjectIDs.h"
#include "AAFResult.h"
#include "ModuleTest.h"
#include "AAFDefUIDs.h"

#include "CAAFBuiltinDefs.h"


// default test values
#define kStoredHeightTestVal			248
#define kStoredWidthTestVal				720
#define kFrameLayoutTestVal				kAAFSeparateFields
#define kVideoLineMapMaxElement			2
#define kVideoLineMap1TestVal			10
#define kVideoLineMap2TestVal			11
#define kImageAspectRatioNumTestVal		4
#define kImageAspectRatioDenTestVal		3
#define kSampledHeightTestVal			247
#define kSampledWidthTestVal			719
#define kSampledXOffsetTestVal			5
#define kSampledYOffsetTestVal			6
#define kDisplayHeightTestVal			246
#define kDisplayWidthTestVal			718
#define kDisplayXOffsetTestVal			7
#define kDisplayYOffsetTestVal			8
#define kAlphaTransparencyTestVal		kAAFMaxValueTransparent
#define kImageAlignmentFactorTestVal	0
#define kFieldDominanceTestVal	kAAFFieldOne
#define kFieldStartOffsetTestVal		128
#define kFieldEndOffsetTestVal			512
#define kDisplayF2OffsetTestVal			1
#define kStoredF2OffsetTestVal			-1
#define kActiveFormatDescriptorTestVal	3
#define kSignalStandardTestVal			kAAFSignalStandard_ITU1358
#define kSignalStandardTestValCase2		kAAFSignalStandard_SMPTE428_1

static const 	aafMobID_t	TEST_MobID =
{{0x06, 0x0c, 0x2b, 0x34, 0x02, 0x05, 0x11, 0x01, 0x01, 0x00, 0x10, 0x00},
0x13, 0x00, 0x00, 0x00,
{0x4b8a7f32, 0x03fe, 0x11d4, {0x8e, 0x3d, 0x00, 0x90, 0x27, 0xdf, 0xca, 0x7c}}};

static const 	aafMobID_t	TEST_MobIDCase2 =
{{0x06, 0x0c, 0x2b, 0x34, 0x02, 0x05, 0x11, 0x01, 0x01, 0x00, 0x10, 0x00},
0x13, 0x00, 0x00, 0x00,
{0x4b8a7f32, 0x03fe, 0x11d4, {0x8e, 0x3d, 0x00, 0x90, 0x27, 0xdf, 0xca, 0x7d}}};

static const aafUID_t kGammaTestVal =
{0x04010101, 0x0101, 0x0000, { 0x06, 0x0E, 0x2B, 0x34, 0x04, 0x01, 0x01, 0x01}};

static const aafUID_t kTransferCharacteristicTestVal =
{0x04010101, 0x0102, 0x0000, { 0x06, 0x0E, 0x2B, 0x34, 0x04, 0x01, 0x01, 0x01}};

static const aafUID_t kColorPrimariesTestVal =
{0x04010101, 0x0301, 0x0000, { 0x06, 0x0E, 0x2B, 0x34, 0x04, 0x01, 0x01, 0x06}};

static const aafUID_t kCodingEquationsTestVal =
{0x04010101, 0x0201, 0x0000, { 0x06, 0x0E, 0x2B, 0x34, 0x04, 0x01, 0x01, 0x01}};

// convenient error handlers.
inline void checkResult(HRESULT r)
{
  if (FAILED(r))
    throw r;
}
inline void checkExpression(bool expression, HRESULT r)
{
  if (!expression)
    throw r;
}


static HRESULT CreateAAFFile(
    aafWChar * pFileName,
    aafUID_constref fileKind,
    testRawStorageType_t rawStorageType,
    aafProductIdentification_constref productID)
{
	IAAFFile*		pFile = NULL;
	IAAFHeader*		pHeader = NULL;
	IAAFDictionary*	pDictionary = NULL;
	IAAFSourceMob*	pSourceMob = NULL;
	IAAFSourceMob*	pSourceMobCase2 = NULL;
	IAAFMob*	pMob = NULL;
	IAAFMob*	pMobCase2 = NULL;
	IAAFDigitalImageDescriptor*	pDIDesc = NULL;
	IAAFDigitalImageDescriptor*	pDIDescCase2 = NULL;
	IAAFDigitalImageDescriptor2*	pDIDesc2 = NULL;
	IAAFDigitalImageDescriptor2*	pDIDesc2Case2 = NULL;
	IAAFEssenceDescriptor*	pEssDesc = NULL;
	IAAFEssenceDescriptor*	pEssDescCase2 = NULL;
	HRESULT			hr = AAFRESULT_SUCCESS;


  try
  {  
    // Remove the previous test file if any.
    RemoveTestFile(pFileName);

    // Create the AAF file
    checkResult(CreateTestFile( pFileName, fileKind, rawStorageType, productID, &pFile ));

    // Get the AAF file header.
    checkResult(pFile->GetHeader(&pHeader));

    // Get the AAF Dictionary so that we can create valid AAF objects.
    checkResult(pHeader->GetDictionary(&pDictionary));
	CAAFBuiltinDefs defs (pDictionary);

    // Create a source mob
    checkResult(defs.cdSourceMob()->
				CreateInstance(IID_IAAFSourceMob, 
							   (IUnknown **)&pSourceMob));

    checkResult(pSourceMob->QueryInterface(IID_IAAFMob, (void **)&pMob));

    checkResult(pMob->SetMobID(TEST_MobID));
    checkResult(pMob->SetName(L"DigitalImageDescriptorTest"));


    // Create a concrete subclass of DigitialImageDescriptor.
    checkResult(defs.cdRGBADescriptor()->
				CreateInstance(IID_IAAFDigitalImageDescriptor, 
							   (IUnknown **)&pDIDesc));		

    aafRational_t	ratio;
    aafInt32		VideoLineMap[kVideoLineMapMaxElement] = {kVideoLineMap1TestVal,kVideoLineMap2TestVal};
    aafUID_t		compression;
    memset(&compression, 0, sizeof(aafUID_t));

    // Add all DigitalImage properties
    // Reguired Properties
    checkResult(pDIDesc->SetStoredView(kStoredHeightTestVal, kStoredWidthTestVal));
    checkResult(pDIDesc->SetFrameLayout(kFrameLayoutTestVal));
    checkResult(pDIDesc->SetVideoLineMap(kVideoLineMapMaxElement, VideoLineMap));

    ratio.numerator = kImageAspectRatioNumTestVal;
    ratio.denominator = kImageAspectRatioDenTestVal;
    checkResult(pDIDesc->SetImageAspectRatio(ratio));

    // Optional Properties
    checkResult(pDIDesc->SetCompression(compression));
    checkResult(pDIDesc->SetSampledView(kSampledHeightTestVal, kSampledWidthTestVal, kSampledXOffsetTestVal, kSampledYOffsetTestVal));
    checkResult(pDIDesc->SetDisplayView(kDisplayHeightTestVal, kDisplayWidthTestVal, kDisplayXOffsetTestVal, kDisplayYOffsetTestVal));
    checkResult(pDIDesc->SetAlphaTransparency(kAlphaTransparencyTestVal));
    checkResult(pDIDesc->SetImageAlignmentFactor(kImageAlignmentFactorTestVal));
    checkResult(pDIDesc->SetGamma(kGammaTestVal));

    // Optional Properties accessed using IAAFDigitalImageDescriptor2
	checkResult(pDIDesc->QueryInterface (IID_IAAFDigitalImageDescriptor2, (void **)&pDIDesc2));

	// DID2::SetTransferCharacteristic() sets the same property as DID::SetGamma()
    checkResult(pDIDesc2->SetTransferCharacteristic(kTransferCharacteristicTestVal));

    checkResult(pDIDesc2->SetColorPrimaries(kColorPrimariesTestVal));
    checkResult(pDIDesc2->SetCodingEquations(kCodingEquationsTestVal));
    checkResult(pDIDesc2->SetFieldDominance(kFieldDominanceTestVal));
    checkResult(pDIDesc2->SetFieldStartOffset(kFieldStartOffsetTestVal));
    checkResult(pDIDesc2->SetFieldEndOffset(kFieldEndOffsetTestVal));

    checkResult(pDIDesc2->SetDisplayF2Offset(kDisplayF2OffsetTestVal));
    checkResult(pDIDesc2->SetStoredF2Offset(kStoredF2OffsetTestVal));
    checkResult(pDIDesc2->SetActiveFormatDescriptor(kActiveFormatDescriptorTestVal));
    checkResult(pDIDesc2->SetSignalStandard(kSignalStandardTestVal));

    // Save the initialized descriptor with the source mob.
    checkResult(pDIDesc->QueryInterface(IID_IAAFEssenceDescriptor, (void **)&pEssDesc));
    checkResult(pSourceMob->SetEssenceDescriptor(pEssDesc));

    // Add the MOB to the file
    checkResult(pHeader->AddMob(pMob));

	// Case 2

	// Create a source mob
    checkResult(defs.cdSourceMob()->
				CreateInstance(IID_IAAFSourceMob, 
							   (IUnknown **)&pSourceMobCase2));

    checkResult(pSourceMobCase2->QueryInterface(IID_IAAFMob, (void **)&pMobCase2));

    checkResult(pMobCase2->SetMobID(TEST_MobIDCase2));
    checkResult(pMobCase2->SetName(L"DigitalImageDescriptorTest Case2"));

	// Create a concrete subclass of DigitialImageDescriptor.
	checkResult(defs.cdRGBADescriptor()->
		CreateInstance(IID_IAAFDigitalImageDescriptor, 
		(IUnknown **)&pDIDescCase2));		

	// Add all DigitalImage properties
	// Required Properties
	checkResult(pDIDescCase2->SetStoredView(kStoredHeightTestVal, kStoredWidthTestVal));
	checkResult(pDIDescCase2->SetFrameLayout(kFrameLayoutTestVal));
	checkResult(pDIDescCase2->SetVideoLineMap(kVideoLineMapMaxElement, VideoLineMap));

	// Optional Properties accessed using IAAFDigitalImageDescriptor2
	checkResult(pDIDescCase2->QueryInterface (IID_IAAFDigitalImageDescriptor2, (void **)&pDIDesc2Case2));

	checkResult(pDIDesc2Case2->SetSignalStandard(kSignalStandardTestValCase2));

	// Save the initialized descriptor with the source mob.
	checkResult(pDIDescCase2->QueryInterface(IID_IAAFEssenceDescriptor, (void **)&pEssDescCase2));
	checkResult(pSourceMobCase2->SetEssenceDescriptor(pEssDescCase2));

	// Add the MOB to the file
	checkResult(pHeader->AddMob(pMobCase2));

  }
  catch (HRESULT& rResult)
  {
    hr = rResult;
  }

  // Cleanup and return
  if (pEssDesc)
    pEssDesc->Release();

  if (pEssDescCase2)
	  pEssDescCase2->Release();

  if (pDIDesc2)
    pDIDesc2->Release();

  if (pDIDesc2Case2)
	  pDIDesc2Case2->Release();

  if (pDIDesc)
    pDIDesc->Release();

  if (pDIDescCase2)
	  pDIDescCase2->Release();

  if (pMob)
    pMob->Release();

  if (pMobCase2)
	  pMobCase2->Release();

  if (pSourceMob)
    pSourceMob->Release();

  if (pSourceMobCase2)
	  pSourceMobCase2->Release();

	if (pDictionary)
    pDictionary->Release();

	if (pHeader)
    pHeader->Release();

	if (pFile)
	{
		pFile->Save();
		pFile->Close();
		pFile->Release();
	}

	return hr;
}

static HRESULT ReadAAFFile(aafWChar * pFileName)
{
	IAAFFile*		pFile = NULL;
	IAAFHeader*		pHeader = NULL;
	IEnumAAFMobs*	pMobIter = NULL;
	IAAFMob*	pMob = NULL;
	IAAFMob*	pMobCase2 = NULL;
	IAAFSourceMob*	pSourceMob = NULL;
	IAAFSourceMob*	pSourceMobCase2 = NULL;
	IAAFEssenceDescriptor*	pEssDesc = NULL;
	IAAFEssenceDescriptor*	pEssDescCase2 = NULL;
	IAAFDigitalImageDescriptor*	pDIDesc = NULL;
	IAAFDigitalImageDescriptor2*	pDIDesc2 = NULL;
	IAAFDigitalImageDescriptor2*	pDIDesc2Case2 = NULL;
	aafNumSlots_t	numMobs = 0;
	HRESULT			hr = AAFRESULT_SUCCESS;


  try
  {
	  // Open the AAF file
	  checkResult(AAFFileOpenExistingRead(pFileName, 0, &pFile));

	  // Get the AAF file header.
	  checkResult(pFile->GetHeader(&pHeader));

    // Make sure there is one a single mob in the file.
	  checkResult(pHeader->CountMobs(kAAFAllMob, &numMobs));
	  checkExpression(2 == numMobs, AAFRESULT_TEST_FAILED);

    // Loop to the first mob.
	  checkResult(pHeader->GetMobs(NULL, &pMobIter));
	  for (int mobNum=0; mobNum < numMobs; ++mobNum)
	  {
		  IAAFMob*	pMobTmp = NULL;
		  checkResult(pMobIter->NextOne(&pMobTmp));

		  aafMobID_t mobID;
		  checkResult(pMobTmp->GetMobID(&mobID));
		  if (memcmp(&mobID, &TEST_MobID, sizeof(aafMobID_t)) == 0)
		  {
			  pMob = pMobTmp;
		  }
		  else if (memcmp(&mobID, &TEST_MobIDCase2, sizeof(aafMobID_t)) == 0)
		  {
			  pMobCase2 = pMobTmp;
		  }
		  else
		  {
			  checkExpression(false, AAFRESULT_TEST_FAILED);
		  }
	  }

	  checkResult(pMob->QueryInterface(IID_IAAFSourceMob, (void **)&pSourceMob));

    // Back into testing mode
	  checkResult(pSourceMob->GetEssenceDescriptor(&pEssDesc));

    // if there is an Essence Descriptor then it MUST be an (essence) DigitalImage Descriptor
	  checkResult(pEssDesc->QueryInterface(IID_IAAFDigitalImageDescriptor, (void **) &pDIDesc));

    // TODO: test for expected DigitalImage properties
	  aafUInt32				val1, val2, iaf;
	  aafInt32				val3, val4;
	  aafFrameLayout_t		framelayout;
	  aafAlphaTransparency_t	alphaTrans;
	  aafRational_t			ratio;
	  aafUID_t				gamma;
	  aafInt32				VideoLineMap[kVideoLineMapMaxElement];
	  aafUID_t				compression, compTestVal;
	  aafUID_t				transferChar;
	  aafUID_t				colorPrimaries;
	  aafUID_t				codingEquations;
		aafFieldNumber_t fieldDominance = kAAFUnspecifiedField;
	  aafUInt32				fieldStartOffset = 0;
	  aafUInt32				fieldEndOffset = 0;
	  aafInt32				displayF2Offset = 0;
	  aafInt32				storedF2Offset = 0;
	  aafUInt8				activeFormatDescriptor = 0;
	  aafSignalStandard_t signalStandard = kAAFSignalStandard_None;

	  memset(&compTestVal, 0, sizeof(aafUID_t));

	  // Add all DigitalImage properties
	  // Reguired Properties
	  checkResult(pDIDesc->GetStoredView(&val1, &val2));
	  checkExpression(val1 == kStoredHeightTestVal && val2 == kStoredWidthTestVal,
						        AAFRESULT_TEST_FAILED);

	  checkResult(pDIDesc->GetFrameLayout(&framelayout));
	  checkExpression(framelayout == kFrameLayoutTestVal, AAFRESULT_TEST_FAILED);

	  checkResult(pDIDesc->GetVideoLineMapSize(&val1));
	  checkExpression(val1 == kVideoLineMapMaxElement, AAFRESULT_TEST_FAILED);

	  checkResult(pDIDesc->GetVideoLineMap(kVideoLineMapMaxElement, VideoLineMap));
	  checkExpression(VideoLineMap[0] == kVideoLineMap1TestVal && 
                    VideoLineMap[1] == kVideoLineMap2TestVal,
                    AAFRESULT_TEST_FAILED);
						  
	  checkResult(pDIDesc->GetImageAspectRatio(&ratio));
    checkExpression(ratio.numerator == kImageAspectRatioNumTestVal &&
                    ratio.denominator == kImageAspectRatioDenTestVal,
                    AAFRESULT_TEST_FAILED);

	  // Optional Properties
	  checkResult(pDIDesc->GetCompression(&compression));
	  checkExpression(memcmp(&compression, &compTestVal, sizeof(aafUID_t)) == 0,
                    AAFRESULT_TEST_FAILED);

	  checkResult(pDIDesc->GetSampledView(&val1, &val2, &val3, &val4));
	  checkExpression(val1 == kSampledHeightTestVal && val2 == kSampledWidthTestVal &&
                    val3 == kSampledXOffsetTestVal && val4 == kSampledYOffsetTestVal,
                    AAFRESULT_TEST_FAILED);

		checkResult(pDIDesc->GetDisplayView(&val1, &val2, &val3, &val4));
		checkExpression(val1 == kDisplayHeightTestVal && val2 == kDisplayWidthTestVal &&
                    val3 == kDisplayXOffsetTestVal && val4 == kDisplayYOffsetTestVal,
                    AAFRESULT_TEST_FAILED);

		checkResult(pDIDesc->GetAlphaTransparency(&alphaTrans));
		checkExpression(alphaTrans == kAlphaTransparencyTestVal,
                    AAFRESULT_TEST_FAILED);

		checkResult(pDIDesc->GetImageAlignmentFactor(&iaf));
		checkExpression(iaf == kImageAlignmentFactorTestVal,
                    AAFRESULT_TEST_FAILED);

		checkResult(pDIDesc->GetGamma(&gamma));
		checkExpression(memcmp(&gamma, &kTransferCharacteristicTestVal, sizeof(kTransferCharacteristicTestVal)) == 0,
                    AAFRESULT_TEST_FAILED);

    // Optional Properties accessed using IAAFDigitalImageDescriptor2
		checkResult(pDIDesc->QueryInterface (IID_IAAFDigitalImageDescriptor2, (void **)&pDIDesc2));

    checkResult(pDIDesc2->GetTransferCharacteristic(&transferChar));
		checkExpression(memcmp(&transferChar, &kTransferCharacteristicTestVal, sizeof(kTransferCharacteristicTestVal)) == 0,
                    AAFRESULT_TEST_FAILED);

    checkResult(pDIDesc2->GetColorPrimaries(&colorPrimaries));
		checkExpression(memcmp(&colorPrimaries, &kColorPrimariesTestVal, sizeof(kColorPrimariesTestVal)) == 0,
                    AAFRESULT_TEST_FAILED);

    checkResult(pDIDesc2->GetCodingEquations(&codingEquations));
		checkExpression(memcmp(&codingEquations, &kCodingEquationsTestVal, sizeof(kCodingEquationsTestVal)) == 0,
                    AAFRESULT_TEST_FAILED);

    checkResult(pDIDesc2->GetFieldDominance(&fieldDominance));
		checkExpression(fieldDominance == kFieldDominanceTestVal, AAFRESULT_TEST_FAILED);

    checkResult(pDIDesc2->GetFieldStartOffset(&fieldStartOffset));
		checkExpression(fieldStartOffset == kFieldStartOffsetTestVal, AAFRESULT_TEST_FAILED);

    checkResult(pDIDesc2->GetFieldEndOffset(&fieldEndOffset));
		checkExpression(fieldEndOffset == kFieldEndOffsetTestVal, AAFRESULT_TEST_FAILED);




    checkResult(pDIDesc2->GetDisplayF2Offset(&displayF2Offset));
		checkExpression(displayF2Offset == kDisplayF2OffsetTestVal, AAFRESULT_TEST_FAILED);

    checkResult(pDIDesc2->GetStoredF2Offset(&storedF2Offset));
		checkExpression(storedF2Offset == kStoredF2OffsetTestVal, AAFRESULT_TEST_FAILED);

    checkResult(pDIDesc2->GetActiveFormatDescriptor(&activeFormatDescriptor));
		checkExpression(activeFormatDescriptor == kActiveFormatDescriptorTestVal, AAFRESULT_TEST_FAILED);

    checkResult(pDIDesc2->GetSignalStandard(&signalStandard));
		checkExpression(signalStandard == kSignalStandardTestVal, AAFRESULT_TEST_FAILED);


	// Case 2
	  checkResult(pMobCase2->QueryInterface(IID_IAAFSourceMob, (void **)&pSourceMobCase2));

    // Back into testing mode
	  checkResult(pSourceMobCase2->GetEssenceDescriptor(&pEssDescCase2));

	  // Optional Properties accessed using IAAFDigitalImageDescriptor2
	  checkResult(pEssDescCase2->QueryInterface (IID_IAAFDigitalImageDescriptor2, (void **)&pDIDesc2Case2));

	  checkResult(pDIDesc2Case2->GetSignalStandard(&signalStandard));
	  checkExpression(signalStandard == kSignalStandardTestValCase2, AAFRESULT_TEST_FAILED);
  }
  catch (HRESULT& rResult)
  {
    hr = rResult;
  }

  // Cleanup and return
  if (pEssDesc)
    pEssDesc->Release();

  if (pEssDescCase2)
	  pEssDescCase2->Release();

  if (pDIDesc2)
    pDIDesc2->Release();

  if (pDIDesc2Case2)
	  pDIDesc2Case2->Release();

  if (pDIDesc)
    pDIDesc->Release();

  if (pMob)
    pMob->Release();

  if (pMobCase2)
	  pMobCase2->Release();

  if (pMobIter)
    pMobIter->Release();

  if (pSourceMob)
    pSourceMob->Release();

  if (pSourceMobCase2)
	  pSourceMobCase2->Release();

	if (pHeader)
    pHeader->Release();

	if (pFile)
	{
		pFile->Close();
		pFile->Release();
	}

	return hr;
}

extern "C" HRESULT CAAFDigitalImageDescriptor_test(
    testMode_t mode,
    aafUID_t fileKind,
    testRawStorageType_t rawStorageType,
    aafProductIdentification_t productID)
{
	HRESULT		hr = AAFRESULT_NOT_IMPLEMENTED;
	const size_t fileNameBufLen = 128;
	aafWChar pFileName[ fileNameBufLen ] = L"";
	GenerateTestFileName( productID.productName, fileKind, fileNameBufLen, pFileName );

	try
	{
		if(mode == kAAFUnitTestReadWrite)
			hr = CreateAAFFile(pFileName, fileKind, rawStorageType, productID);
		else
			hr = AAFRESULT_SUCCESS;
		if (SUCCEEDED(hr))
			hr = ReadAAFFile(pFileName);
	}
	catch (...)
	{
		cerr << "CAAFDigitalImageDescriptor_test..."
			 << "Caught general C++ exception!" << endl; 
		hr = AAFRESULT_TEST_FAILED;
	}

	return hr;
}

