// @doc INTERNAL
// @com This file implements the module test for CAAFHeader.
//=---------------------------------------------------------------------=
//
// $Id$ $Name$
//
// The contents of this file are subject to the AAF SDK Public Source
// License Agreement Version 2.0 (the "License"); You may not use this
// file except in compliance with the License.  The License is available
// in AAFSDKPSL.TXT, or you may obtain a copy of the License from the
// Advanced Media Workflow Association, Inc., or its successor.
//
// Software distributed under the License is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See
// the License for the specific language governing rights and limitations
// under the License.  Refer to Section 3.3 of the License for proper use
// of this Exhibit.
//
// WARNING:  Please contact the Advanced Media Workflow Association,
// Inc., for more information about any additional licenses to
// intellectual property covering the AAF Standard that may be required
// to create and distribute AAF compliant products.
// (http://www.amwa.tv/policies).
//
// Copyright Notices:
// The Original Code of this file is Copyright 1998-2009, licensor of the
// Advanced Media Workflow Association.  All rights reserved.
//
// The Initial Developer of the Original Code of this file and the
// licensor of the Advanced Media Workflow Association is
// Avid Technology.
// All rights reserved.
//
//=---------------------------------------------------------------------=

#include "AAF.h"

#include <iostream>
using namespace std;
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include "AAFStoredObjectIDs.h"
#include "AAFResult.h"
#include "ModuleTest.h"
#include "AAFDefUIDs.h"
#include "AAFWideString.h"

#include "CAAFBuiltinDefs.h"


const aafUInt32 gMaxMobCount = 5;

//--cf  //5 mobid's need to be genererated
static const aafMobID_t  _mobID[gMaxMobCount] = 
{//start mobid block
	//first id
	{{0x06, 0x0c, 0x2b, 0x34, 0x02, 0x05, 0x11, 0x01, 0x01, 0x00, 0x10, 0x00},
		0x13, 0x00, 0x00, 0x00,
	{0xca9af110, 0x0400, 0x11d4, {0x8e, 0x3d, 0x00, 0x90, 0x27, 0xdf, 0xca, 0x7c}}},
	
	//second id
	{{0x06, 0x0c, 0x2b, 0x34, 0x02, 0x05, 0x11, 0x01, 0x01, 0x00, 0x10, 0x00},
	0x13, 0x00, 0x00, 0x00,
	{0xe00691a8, 0x0400, 0x11d4, {0x8e, 0x3d, 0x00, 0x90, 0x27, 0xdf, 0xca, 0x7c}}},
	
	//third id
	{{0x06, 0x0c, 0x2b, 0x34, 0x02, 0x05, 0x11, 0x01, 0x01, 0x00, 0x10, 0x00},
	0x13, 0x00, 0x00, 0x00,
	{0xea00082e, 0x0400, 0x11d4, {0x8e, 0x3d, 0x00, 0x90, 0x27, 0xdf, 0xca, 0x7c}}},
	
	//fourth id
	{{0x06, 0x0c, 0x2b, 0x34, 0x02, 0x05, 0x11, 0x01, 0x01, 0x00, 0x10, 0x00},
	0x13, 0x00, 0x00, 0x00,
	{0xf6d0abee, 0x0400, 0x11d4, {0x8e, 0x3d, 0x00, 0x90, 0x27, 0xdf, 0xca, 0x7c}}},
	
	//fifth id
	{{0x06, 0x0c, 0x2b, 0x34, 0x02, 0x05, 0x11, 0x01, 0x01, 0x00, 0x10, 0x00},
	0x13, 0x00, 0x00, 0x00,
	{0x002f299a, 0x0401, 0x11d4, {0x8e, 0x3d, 0x00, 0x90, 0x27, 0xdf, 0xca, 0x7c}}}
	
}; //end mobid block

const aafUID_t setOpPatternID = {0x4e8404ff, 0x0f29, 0x11f4, {0xf3, 0x59, 0x0f, 0x90, 0x27, 0xdf, 0xca, 0x6a}};


// Descriptiove scheme IDs to be added to the Header
static const aafUID_t  descriptiveSchemes[] =
{
    // {93C0C960-8C89-4050-81AA-4B540C372A54}
    { 0x93c0c960, 0x8c89, 0x4050, { 0x81, 0xaa, 0x4b, 0x54, 0x0c, 0x37, 0x2a, 0x54 } },
    // {A731B402-D563-4de8-B2E8-1A812A3F6F98}
    { 0xa731b402, 0xd563, 0x4de8, { 0xb2, 0xe8, 0x1a, 0x81, 0x2a, 0x3f, 0x6f, 0x98 } },
    // {B8BFA1BC-A605-4a62-AA79-A430CC9DE4B3}
    { 0xb8bfa1bc, 0xa605, 0x4a62, { 0xaa, 0x79, 0xa4, 0x30, 0xcc, 0x9d, 0xe4, 0xb3 } }
};

const aafUInt32 gMaxDescriptiveSchemesCount =
                    sizeof(descriptiveSchemes) / sizeof(descriptiveSchemes[0]);


// Descriptiove scheme IDs to be added to and then removed from the Header
static const aafUID_t  tempDescriptiveSchemes[] =
{
    // {A9020C2F-0659-4cb0-8135-5AB3D9E8B793}
    { 0xa9020c2f, 0x0659, 0x4cb0, { 0x81, 0x35, 0x5a, 0xb3, 0xd9, 0xe8, 0xb7, 0x93 } },
    // {F69ABEF0-A871-41cf-BC7C-F5763CA22E53}
    { 0xf69abef0, 0xa871, 0x41cf, { 0xbc, 0x7c, 0xf5, 0x76, 0x3c, 0xa2, 0x2e, 0x53 } },
    // {DB6305D3-092C-4c31-9A7E-F8D558E8035E}
    { 0xdb6305d3, 0x092c, 0x4c31, { 0x9a, 0x7e, 0xf8, 0xd5, 0x58, 0xe8, 0x03, 0x5e } }
};

const aafUInt32 gMaxTempDescriptiveSchemesCount =
                    sizeof(tempDescriptiveSchemes) / sizeof(tempDescriptiveSchemes[0]);


// {B8CF441F-F2F5-11d4-8040-00104BC9156D}
static const aafUID_t Header_OptionalPropID =
{ 0xb8cf441f, 0xf2f5, 0x11d4, { 0x80, 0x40, 0x0, 0x10, 0x4b, 0xc9, 0x15, 0x6d } };

static aafCharacter_constptr Header_OptionalPropName = L"Header optional property";

static const aafUInt32 Header_OptionalPropertyValue = 42;

inline void checkhr(HRESULT expression, HRESULT r)
{
  if (expression != r)
    throw r;
}

// Utility class to implement the test.
struct HeaderTest
{
  HeaderTest(aafProductIdentification_constref productID);
  ~HeaderTest();

  void createEmptyDMSchemesSetFile(wchar_t *pFileName,
                  aafUID_constref fileKind,
                  testRawStorageType_t rawStorageType);
  void openEmptyDMSchemesSetFile(wchar_t *pFileName);

  void createFile(wchar_t *pFileName,
                  aafUID_constref fileKind,
                  testRawStorageType_t rawStorageType);
  void openFile(wchar_t *pFileName);

  void writeDescriptiveSchemes();
  void readDescriptiveSchemes();

  void writeEmptyDescriptiveSchemes();
  void readEmptyDescriptiveSchemes();

  void writeOptionalProperties();
  void readOptionalProperties();

  void checkByteOrder();
  void checkOperationalPattern();
  void checkEssenceContainer();
  void createFileMob(aafUInt32 itemNumber);
  void createEssenceData(IAAFSourceMob *pSourceMob);
  void openMobs();
  void openEssenceData();

  void setPrimaryMob(aafUInt32 itemNumber);
  void readPrimaryMob(aafUInt32 itemNumber);

  int formatMobName(aafUInt32 itemNumber, wchar_t* mobName);
  aafUID_t getMobContainerFormatID(aafUInt32 itemNumber);
  void cleanupReferences();
  void check(HRESULT hr);
  void removeTestFile(const wchar_t* pFileName);

  // Shared member data:
  HRESULT _hr;
  aafProductVersion_t _productVersion;
  aafProductIdentification_t _productInfo;
  IAAFFile *_pFile;
  bool _bFileOpen;
  IAAFHeader *_pHeader;
  IAAFHeader2 *_pHeader2;
  IAAFDictionary *_pDictionary;
  IAAFDictionary2 *_pDictionary2;

  IAAFMob *_pMob;
  IAAFSourceMob *_pSourceMob;
  IEnumAAFMobs *_pEnumMobs;
  IAAFEssenceDescriptor *_pEssenceDescriptor;
  IAAFFileDescriptor *_pFileDescriptor;
  IEnumAAFEssenceData *_pEnumEssenceData;
  IAAFEssenceData *_pEssenceData;
  IAAFIdentification *_pIdent;
  IEnumAAFIdentifications *_pEnumIdent;
  IAAFObject *_pHeaderObject;
  IAAFClassDef *_pClassDef;
  IAAFTypeDef *_pOptionalPropTypeDef;
  IAAFTypeDefInt *_pTypeDefInt;
  IAAFPropertyDef *_pOptionalPropDef;
  IAAFPropertyValue *_pOptionalPropValue;
  IAAFEndian *_pEndian;
};

extern "C" HRESULT CAAFHeader_test(
    testMode_t mode,
    aafUID_t fileKind,
    testRawStorageType_t rawStorageType,
    aafProductIdentification_t productID);
extern "C" HRESULT CAAFHeader_test(
    testMode_t mode,
    aafUID_t fileKind,
    testRawStorageType_t rawStorageType,
    aafProductIdentification_t productID)
{
  HRESULT hr = AAFRESULT_SUCCESS;
  const size_t fileNameBufLen = 128;
  aafWChar fileName[ fileNameBufLen ] = L"";
  GenerateTestFileName( productID.productName, fileKind, fileNameBufLen, fileName );
  HeaderTest ht(productID);

  aafWChar nameExtension[] = L"Empty";
  size_t strSize = wcslen(productID.productName) + wcslen(nameExtension) + 1;//For the '\0'
  aafWChar *productName2 = new aafWChar[strSize];  
  wcscpy(productName2, productID.productName);
  wcscat(productName2, nameExtension);
  aafWChar fileName2[ fileNameBufLen  ] = L"";
  GenerateTestFileName( productName2, fileKind, fileNameBufLen, fileName2 );
  delete [] productName2; productName2 = 0;
  

  try
  {
	if(mode == kAAFUnitTestReadWrite)
	   ht.createFile(fileName, fileKind, rawStorageType);
    ht.openFile(fileName);

	if(!isSSFileKind(EffectiveTestFileEncoding(fileKind)))
	{
		if(mode == kAAFUnitTestReadWrite)
			ht.createEmptyDMSchemesSetFile(fileName2, fileKind, rawStorageType);
		ht.openEmptyDMSchemesSetFile(fileName2);
	}
  }
  catch (HRESULT& ehr)
  {
    // thrown by HeaderTest_check() method.
    hr = ehr;
  }
  catch (...)
  {
    cerr << "CAAFHeader_test...Caught general C++"
		 << " exception!" << endl; 
	hr = AAFRESULT_TEST_FAILED;
  }

  // Cleanup our object if it exists.

  return hr;
}

HeaderTest::HeaderTest(aafProductIdentification_constref productID):
  _hr(AAFRESULT_SUCCESS),
  _productInfo(productID),
  _pFile(NULL),
  _bFileOpen(false),
  _pHeader(NULL),
  _pHeader2(NULL),
  _pDictionary(NULL),
  _pDictionary2(NULL),
  _pMob(NULL),
  _pSourceMob(NULL),
  _pEnumMobs(NULL),
  _pEssenceDescriptor(NULL),
  _pFileDescriptor(NULL),
  _pEnumEssenceData(NULL),
  _pEssenceData(NULL),
  _pHeaderObject(NULL),
  _pClassDef(NULL),
  _pOptionalPropTypeDef(NULL),
  _pTypeDefInt(NULL),
  _pOptionalPropDef(NULL),
  _pOptionalPropValue(NULL),
  _pEndian(NULL)
{
}

HeaderTest::~HeaderTest()
{
  cleanupReferences();
}

void HeaderTest::cleanupReferences()
{
  if (NULL != _pOptionalPropValue)
  {
    _pOptionalPropValue->Release();
    _pOptionalPropValue = NULL;
  }

  if (NULL != _pOptionalPropDef)
  {
    _pOptionalPropDef->Release();
    _pOptionalPropDef = NULL;
  }

  if (NULL != _pTypeDefInt)
  {
    _pTypeDefInt->Release();
    _pTypeDefInt = NULL;
  }

  if (NULL != _pOptionalPropTypeDef)
  {
    _pOptionalPropTypeDef->Release();
    _pOptionalPropTypeDef = NULL;
  }

  if (NULL != _pClassDef)
  {
    _pClassDef->Release();
    _pClassDef = NULL;
  }

  if (NULL != _pHeaderObject)
  {
    _pHeaderObject->Release();
    _pHeaderObject = NULL;
  }

  if (NULL != _pEssenceData)
  {
    _pEssenceData->Release();
    _pEssenceData = NULL;
  }

  if (NULL != _pEnumEssenceData)
  {
    _pEnumEssenceData->Release();
    _pEnumEssenceData = NULL;
  }

  if (NULL != _pFileDescriptor)
  {
    _pFileDescriptor->Release();
    _pFileDescriptor = NULL;
  }

  if (NULL != _pEssenceDescriptor)
  {
    _pEssenceDescriptor->Release();
    _pEssenceDescriptor = NULL;
  }

  if (NULL != _pSourceMob)
  {
    _pSourceMob->Release();
    _pSourceMob = NULL;
  }

  if (NULL != _pMob)
  {
    _pMob->Release();
    _pMob = NULL;
  }

  if (NULL != _pEnumMobs)
  {
    _pEnumMobs->Release();
    _pEnumMobs = NULL;
  }

  if (NULL != _pDictionary)
  {
    _pDictionary->Release();
    _pDictionary = NULL;
  }

  if (NULL != _pHeader)
  {
    _pHeader->Release();
    _pHeader = NULL;
  }

  if (NULL != _pFile)
  {
    if (_bFileOpen)
      _pFile->Close();
    _pFile->Release();
    _pFile = NULL;
  }

  if(NULL != _pEndian)
  {
    _pEndian->Release();
    _pEndian = NULL;
  }

  if(NULL != _pHeader2)
  {
    _pHeader2->Release();
    _pHeader2 = NULL;
  }

  if(NULL != _pDictionary2)
  {
    _pDictionary2->Release();
    _pDictionary2 = NULL;
  }
}

inline void HeaderTest::check(HRESULT hr)
{
  if (AAFRESULT_SUCCESS != (_hr = hr))
    throw hr;
}

// Cross-platform utility to delete a file.
void HeaderTest::removeTestFile(const wchar_t* pFileName)
{
  const size_t kMaxFileName = 512;
  char cFileName[kMaxFileName];

  size_t status = wcstombs(cFileName, pFileName, kMaxFileName);
  if (status != (size_t)-1)
  { // delete the file.
    remove(cFileName);
  }
}

void HeaderTest::createEmptyDMSchemesSetFile(
    wchar_t* pFileName,
    aafUID_constref fileKind,
    testRawStorageType_t rawStorageType)
{
  // Remove the previous test file if any.
  removeTestFile(pFileName);

  check(CreateTestFile( pFileName, fileKind, rawStorageType, _productInfo, &_pFile ));

  _bFileOpen = true;
  check(_pFile->GetHeader(&_pHeader));

  // GetDictionary
  checkhr(_pHeader->GetDictionary(NULL), AAFRESULT_NULL_PARAM);
  check(_pHeader->GetDictionary(&_pDictionary));
  
  CAAFBuiltinDefs defs (_pDictionary);

  writeEmptyDescriptiveSchemes(); 
 
  check(_pFile->Save());
  cleanupReferences();

}

void HeaderTest::openEmptyDMSchemesSetFile(wchar_t *pFileName)
{
  check(AAFFileOpenExistingRead(pFileName, 0, &_pFile));
  _bFileOpen = true;
  check(_pFile->GetHeader(&_pHeader));
  check(_pHeader->GetDictionary(&_pDictionary));

  readEmptyDescriptiveSchemes();

  cleanupReferences();
}

void HeaderTest::createFile(
    wchar_t *pFileName,
    aafUID_constref fileKind,
    testRawStorageType_t rawStorageType)
{
  // Remove the previous test file if any.
  removeTestFile(pFileName);

  check(CreateTestFile( pFileName, fileKind, rawStorageType, _productInfo, &_pFile ));

  _bFileOpen = true;
  check(_pFile->GetHeader(&_pHeader));
  check(_pHeader->QueryInterface(IID_IAAFHeader2, reinterpret_cast<void**>(&_pHeader2)));


  // GetDictionary
  checkhr(_pHeader->GetDictionary(NULL), AAFRESULT_NULL_PARAM);
  check(_pHeader->GetDictionary(&_pDictionary));
  check(_pDictionary->QueryInterface(IID_IAAFDictionary2, reinterpret_cast<void**>(&_pDictionary2)));

  
  CAAFBuiltinDefs defs (_pDictionary);

  // GetRefImplVersion  
  aafProductVersion_t	productVerison;
  checkhr(_pHeader->GetRefImplVersion(NULL), AAFRESULT_NULL_PARAM);
  check(_pHeader->GetRefImplVersion(&productVerison));

  // GetFileRevision  
  aafVersionType_t	fileVersion;
  checkhr(_pHeader->GetFileRevision(NULL), AAFRESULT_NULL_PARAM);
  check(_pHeader->GetFileRevision(&fileVersion));

  // GetLastModified  
  aafTimeStamp_t	timeStamp;
  checkhr(_pHeader->GetLastModified(NULL), AAFRESULT_NULL_PARAM);
  check(_pHeader->GetLastModified(&timeStamp));

  // CountIdentifications
  aafUInt32	numIdents = 0;
  checkhr(_pHeader->CountIdentifications(NULL), AAFRESULT_NULL_PARAM);
  check(_pHeader->CountIdentifications(&numIdents));
  checkhr(1 == numIdents, true);
  
  // GetLastIdentification
  checkhr(_pHeader->GetLastIdentification(NULL), AAFRESULT_NULL_PARAM);
  check(_pHeader->GetLastIdentification(&_pIdent));
  _pIdent->Release();
  _pIdent = NULL;
  
  // AppendIdentification
  check(defs.cdIdentification()->
		CreateInstance(IID_IAAFIdentification, 
					   (IUnknown **)&_pIdent));

  // {319EFBBF-1189-11d4-804A-080036210804}
  static const  aafUID_t productId = 
  { 0x319efbbf, 0x1189, 0x11d4, { 0x80, 0x4a, 0x8, 0x0, 0x36, 0x21, 0x8, 0x4 } };

  check(_pIdent->Initialize(L"Company Name", L"Product Name", L"Product Version String", productId));

  checkhr(_pHeader->AppendIdentification(NULL), AAFRESULT_NULL_PARAM);
  check(_pHeader->AppendIdentification(_pIdent));
  check(_pHeader->CountIdentifications(&numIdents));
  checkhr(2 == numIdents, true);  
  _pIdent->Release();
  _pIdent = NULL;

  // GetIdentificationAt
  checkhr(_pHeader->GetIdentificationAt(0, NULL), AAFRESULT_NULL_PARAM);
  checkhr(_pHeader->GetIdentificationAt(numIdents, &_pIdent), AAFRESULT_BADINDEX);
  check(_pHeader->GetIdentificationAt(0, &_pIdent));
  _pIdent->Release();
  _pIdent = NULL;
  check(_pHeader->GetIdentificationAt(1, &_pIdent));
  _pIdent->Release();
  _pIdent = NULL;

  // GetIdentifications
  checkhr(_pHeader->GetIdentifications(NULL), AAFRESULT_NULL_PARAM);
  check(_pHeader->GetIdentifications(&_pEnumIdent));
  _pEnumIdent->Release();
  _pEnumIdent = NULL;

  aafUID_t generation;
  IAAFObject *pObject = NULL;
  
  // {2865256E-0BEC-11d4-A3FA-0004AC96A937} - guaranteed to *not* match any existing generations
  static const aafUID_t fakeGeneration = 
  	{ 0x2865256e, 0xbec, 0x11d4, { 0xa3, 0xfa, 0x0, 0x4, 0xac, 0x96, 0xa9, 0x37 } };

  // LookupIdentification
  checkhr(_pHeader->LookupIdentification(generation, NULL), AAFRESULT_NULL_PARAM);
  checkhr(_pHeader->LookupIdentification(fakeGeneration, &_pIdent), AAFRESULT_OBJECT_NOT_FOUND);

  check(_pHeader->QueryInterface (IID_IAAFObject, (void **)&pObject));
  check(pObject->EnableGenerationTracking());
  _pFile->Save();  // tjb Hack - cause a generation to be assigned.
  check(pObject->GetGenerationAUID(&generation));
  check(_pHeader->LookupIdentification(generation, &_pIdent));
  _pIdent->Release();
  _pIdent = NULL;
  pObject->Release();
  pObject = NULL;

  // IsEssenceDataPresent
  aafBool 	pResult = true;
  checkhr(_pHeader->IsEssenceDataPresent(_mobID[0], kAAFEssence, NULL), AAFRESULT_NULL_PARAM);  
  checkhr(_pHeader->IsEssenceDataPresent(_mobID[0], 2, &pResult), AAFRESULT_INVALID_ENUM_VALUE);  
  check(_pHeader->IsEssenceDataPresent(_mobID[0], kAAFForeignMedia, &pResult));
  if (pResult) check(AAFRESULT_TEST_FAILED);  
  check(_pHeader->IsEssenceDataPresent(_mobID[0], kAAFEssence, &pResult));  
  if (pResult) check(AAFRESULT_TEST_FAILED);  

  for (aafUInt32 item = 0; item < gMaxMobCount; ++item)
    createFileMob(item);
 

  writeDescriptiveSchemes(); 
  readDescriptiveSchemes();

  writeOptionalProperties();
  readOptionalProperties();

  setPrimaryMob(0);
  readPrimaryMob(0);

  check(_pFile->Save());
  cleanupReferences();
}

void HeaderTest::openFile(wchar_t *pFileName)
{
  check(AAFFileOpenExistingRead(pFileName, 0, &_pFile));
  _bFileOpen = true;
  check(_pFile->GetHeader(&_pHeader));
  check(_pHeader->QueryInterface(IID_IAAFHeader2, reinterpret_cast<void**>(&_pHeader2)));
  check(_pHeader->GetDictionary(&_pDictionary));
  check(_pDictionary->QueryInterface(IID_IAAFDictionary2, reinterpret_cast<void**>(&_pDictionary2)));

  openEssenceData();
  openMobs();
  readDescriptiveSchemes();
  readOptionalProperties();
  readPrimaryMob(0);

  checkByteOrder();
  checkOperationalPattern();
  checkEssenceContainer();
  cleanupReferences();
}

void HeaderTest::writeEmptyDescriptiveSchemes()
{
  assert(_pHeader && _pDictionary);

  IAAFHeader3 *pHeader3 = NULL;
  if (SUCCEEDED(_pHeader->QueryInterface(IID_IAAFHeader3, (void **)&pHeader3)))
  {
	check(pHeader3->CreateEmptyDescriptiveSchemes());
	pHeader3->Release();
    pHeader3 = NULL;
  }
  return;
}

void HeaderTest::readEmptyDescriptiveSchemes()
{
  assert(_pHeader && _pDictionary);

  IAAFHeader3 *pHeader3 = NULL;
  if (SUCCEEDED(_pHeader->QueryInterface(IID_IAAFHeader3, (void **)&pHeader3)))
  {
	aafUInt32 descriptiveSchemesCount = 1;
    check(pHeader3->CountDescriptiveSchemes(&descriptiveSchemesCount));

    if (descriptiveSchemesCount != 0)
	{
		pHeader3->Release();
		pHeader3 = NULL;
        check(AAFRESULT_TEST_FAILED);
	}

	check(pHeader3->RemoveDescriptiveSchemes());

	if(pHeader3->CountDescriptiveSchemes(&descriptiveSchemesCount) != AAFRESULT_PROP_NOT_PRESENT) check(AAFRESULT_TEST_FAILED);

	pHeader3->Release();
    pHeader3 = NULL;
  }
  return;
}

void HeaderTest::writeDescriptiveSchemes()
{
  assert(_pHeader && _pDictionary);

 

  IAAFHeader2 *pHeader2 = NULL;
  if (SUCCEEDED(_pHeader->QueryInterface(IID_IAAFHeader2, (void **)&pHeader2)))
  {
    assert(pHeader2);
    assert(gMaxDescriptiveSchemesCount == gMaxTempDescriptiveSchemesCount);

    for (aafUInt32 i = 0; i < gMaxDescriptiveSchemesCount; ++i)
    {
      check(pHeader2->AddDescriptiveScheme(descriptiveSchemes[i]));
      check(pHeader2->AddDescriptiveScheme(tempDescriptiveSchemes[i]));
    }

    aafUInt32 descriptiveSchemesCount = 0;
    check(pHeader2->CountDescriptiveSchemes(&descriptiveSchemesCount));
    if (descriptiveSchemesCount !=
        (gMaxDescriptiveSchemesCount + gMaxTempDescriptiveSchemesCount))
      check(AAFRESULT_TEST_FAILED);

    for (aafUInt32 j = 0; j < gMaxTempDescriptiveSchemesCount; ++j)
    {
      check(pHeader2->RemoveDescriptiveScheme(tempDescriptiveSchemes[j]));
    }

    descriptiveSchemesCount = 0;
    check(pHeader2->CountDescriptiveSchemes(&descriptiveSchemesCount));
    if (descriptiveSchemesCount != gMaxDescriptiveSchemesCount)
      check(AAFRESULT_TEST_FAILED);

	IAAFHeader3 *pHeader3 = NULL;
	if (SUCCEEDED(_pHeader->QueryInterface(IID_IAAFHeader3, (void **)&pHeader3)))
	{
		if(pHeader3->CreateEmptyDescriptiveSchemes() != AAFRESULT_PROP_ALREADY_PRESENT) check(AAFRESULT_TEST_FAILED);

		pHeader3->Release();
		pHeader3 = NULL;
	}

    pHeader2->Release();
    pHeader2 = NULL;
  }
}


void HeaderTest::readDescriptiveSchemes()
{
  assert(_pHeader && _pDictionary);

  IAAFHeader2 *pHeader2 = NULL;
  if (SUCCEEDED(_pHeader->QueryInterface(IID_IAAFHeader2, (void **)&pHeader2)))
  {
    assert(pHeader2);

    aafUInt32 descriptiveSchemesCount = 0;
    check(pHeader2->CountDescriptiveSchemes(&descriptiveSchemesCount));
    if (descriptiveSchemesCount != gMaxDescriptiveSchemesCount)
      check(AAFRESULT_TEST_FAILED);

    for (aafUInt32 i = 0; i < gMaxDescriptiveSchemesCount; ++i)
    {
      aafBoolean_t descriptiveSchemesPresent = kAAFFalse;
      check(pHeader2->IsDescriptiveSchemePresent(descriptiveSchemes[i],
                                                 &descriptiveSchemesPresent));
      if (!descriptiveSchemesPresent)
        check(AAFRESULT_TEST_FAILED);
    }


    aafUID_t* pDescriptiveSchemesFound = new aafUID_t[descriptiveSchemesCount];
    assert(pDescriptiveSchemesFound);
    check(pHeader2->GetDescriptiveSchemes(descriptiveSchemesCount,
                                          pDescriptiveSchemesFound));
    for (aafUInt32 j = 0; j < gMaxDescriptiveSchemesCount; ++j)
    {
      bool found = false;
      for (aafUInt32 i = 0; i < descriptiveSchemesCount; ++i)
      {
        if (memcmp(descriptiveSchemes+j, pDescriptiveSchemesFound+i, sizeof(aafUID_t)) == 0)
        {
          found = true;
          break;
        }
      }
      if (!found)
        check(AAFRESULT_TEST_FAILED);
    }

    delete[] pDescriptiveSchemesFound;
    pDescriptiveSchemesFound = NULL;
    pHeader2->Release();
    pHeader2 = NULL;
  }
}


void HeaderTest::writeOptionalProperties()
{
  assert(_pHeader && _pDictionary && _pHeader2);

  aafProductVersion_t toolkitVersion;
  check(GetAAFVersions(_pHeader, &toolkitVersion, NULL));

  if (ExtendingAAFObjectSupported(toolkitVersion))
  {
    // First register the new optional property with the class
    // definition for the Header.
    check(_pDictionary->LookupTypeDef(kAAFTypeID_UInt32, &_pOptionalPropTypeDef));
    check(_pOptionalPropTypeDef->QueryInterface(IID_IAAFTypeDefInt, (void **)&_pTypeDefInt));
    check(_pDictionary->LookupClassDef(AUID_AAFHeader, &_pClassDef));
    check(_pClassDef->RegisterOptionalPropertyDef(Header_OptionalPropID, 
                                                  Header_OptionalPropName,
                                                  _pOptionalPropTypeDef,
                                                  &_pOptionalPropDef));
    
    // Now write a value into the new optional property.
    check(_pHeader->QueryInterface(IID_IAAFObject, (void **)&_pHeaderObject));
    check(_pTypeDefInt->CreateValue((aafMemPtr_t)&Header_OptionalPropertyValue,
                                    sizeof(Header_OptionalPropertyValue),
                                    &_pOptionalPropValue));
    check(_pHeaderObject->SetPropertyValue(_pOptionalPropDef, _pOptionalPropValue));

    //test setOperationalPattern method
    if(_pHeader2->SetOperationalPattern(setOpPatternID) != AAFRESULT_SUCCESS)
      check(AAFRESULT_TEST_FAILED);
    
    // Release references
    _pOptionalPropValue->Release();
    _pOptionalPropValue = NULL;
    _pHeaderObject->Release();
    _pHeaderObject = NULL;
    _pOptionalPropDef->Release();
    _pOptionalPropDef = NULL;
    _pClassDef->Release();
    _pClassDef = NULL;
    _pTypeDefInt->Release();
    _pTypeDefInt = NULL;
    _pOptionalPropTypeDef->Release();
    _pOptionalPropTypeDef = NULL;
  }
}


void HeaderTest::readOptionalProperties()
{
  assert(_pHeader && _pDictionary);

  aafProductVersion_t toolkitVersion, fileToolkitVersion;
  check(GetAAFVersions(_pHeader, &toolkitVersion, &fileToolkitVersion));

  if (ExtendingAAFObjectSupported(toolkitVersion) && 
      ExtendingAAFObjectSupported(fileToolkitVersion))
  {
    // First find the new optional property definition in the class
    // definition for the Header.
    check(_pDictionary->LookupTypeDef(kAAFTypeID_UInt32, &_pOptionalPropTypeDef));
    check(_pOptionalPropTypeDef->QueryInterface(IID_IAAFTypeDefInt, (void **)&_pTypeDefInt));
    check(_pDictionary->LookupClassDef(AUID_AAFHeader, &_pClassDef));
    check(_pClassDef->LookupPropertyDef(Header_OptionalPropID, &_pOptionalPropDef));
    
    // Now read a value into the new optional property.
    check(_pHeader->QueryInterface(IID_IAAFObject, (void **)&_pHeaderObject));
    check(_pHeaderObject->GetPropertyValue(_pOptionalPropDef, &_pOptionalPropValue));
    aafUInt32 value = 0;
    check(_pTypeDefInt->GetInteger(_pOptionalPropValue, (aafMemPtr_t)&value, sizeof(value)));
    if (Header_OptionalPropertyValue != value) check(AAFRESULT_TEST_FAILED);

   
    // Release references
    _pOptionalPropValue->Release();
    _pOptionalPropValue = NULL;
    _pHeaderObject->Release();
    _pHeaderObject = NULL;
    _pOptionalPropDef->Release();
    _pOptionalPropDef = NULL;
    _pClassDef->Release();
    _pClassDef = NULL;
    _pTypeDefInt->Release();
    _pTypeDefInt = NULL;
    _pOptionalPropTypeDef->Release();
    _pOptionalPropTypeDef = NULL;
  }
}


int HeaderTest::formatMobName(aafUInt32 itemNumber, wchar_t* mobName)
{
  assert(0 <= itemNumber && gMaxMobCount > itemNumber);
  assert(NULL != mobName);

  char cBuffer[64];

  sprintf(cBuffer, "HeaderTest File Mob %d", itemNumber);
  size_t count = mbstowcs(mobName, cBuffer, strlen(cBuffer) + 1);
  if (static_cast<size_t>(-1) == count)
    check(AAFRESULT_INTERNAL_ERROR);
  return count;
}


void HeaderTest::createFileMob(aafUInt32 itemNumber)
{
  assert(_pFile && _pHeader && _pDictionary && _pDictionary2);
  assert(NULL == _pSourceMob);
  assert(NULL == _pMob);
  assert(NULL == _pFileDescriptor);
  assert(NULL == _pEssenceDescriptor);
  assert(NULL == _pSourceMob);
  assert(0 <= itemNumber && gMaxMobCount > itemNumber);

  // Format the mob name.
  wchar_t wcBuffer[128];
  formatMobName(itemNumber, wcBuffer);

  CAAFBuiltinDefs defs (_pDictionary);

  // Create a Mob
  check(defs.cdSourceMob()->
		CreateInstance(IID_IAAFSourceMob, 
					   (IUnknown **)&_pSourceMob));

  check(_pSourceMob->QueryInterface (IID_IAAFMob, (void **)&_pMob));
  
  check(_pMob->SetMobID(_mobID[itemNumber]));

  check(_pMob->SetName(wcBuffer));
  
  // create concrete subclass of FileDescriptor
  check(defs.cdAIFCDescriptor()->
		CreateInstance(IID_IAAFFileDescriptor, 
					   (IUnknown **)&_pFileDescriptor));


	IAAFAIFCDescriptor*			pAIFCDesc = NULL;
	check(_pFileDescriptor->QueryInterface (IID_IAAFAIFCDescriptor, (void **)&pAIFCDesc));
	check(pAIFCDesc->SetSummary (5, (unsigned char*)"TEST"));
	pAIFCDesc->Release();
	pAIFCDesc = NULL;

  check(_pFileDescriptor->QueryInterface (IID_IAAFEssenceDescriptor,
                                          (void **)&_pEssenceDescriptor));
  check(_pSourceMob->SetEssenceDescriptor (_pEssenceDescriptor));
  

	//create a container for the AIFCDescriptor, assigning it a container definition
	//4 containers will be created
	aafUInt32 num;
	check(_pDictionary2->CountContainerDefs(&num));

	IEnumAAFContainerDefs *pEnum;
	IAAFContainerDef *pDef;

	check(_pDictionary2->GetContainerDefs(&pEnum));

	for(unsigned int i = 0; i < itemNumber && i < num; i++)
	{
		check(pEnum->NextOne(&pDef));
		pDef->Release();
		pDef = NULL;
	}

	check(pEnum->NextOne(&pDef));
	check(pDef->SetEssenceIsIdentified((aafBoolean_t)true));
	check(_pFileDescriptor->SetContainerFormat(pDef));
	pDef->Release();
	pDef = NULL;

	pEnum->Release();
	pEnum = NULL;

  // AddMob
  checkhr(_pHeader->AddMob(NULL), AAFRESULT_NULL_PARAM);
  check(_pHeader->AddMob(_pMob));
  checkhr(_pHeader->AddMob(_pMob), AAFRESULT_DUPLICATE_MOBID);
  
  // RemoveMob
  checkhr(_pHeader->RemoveMob(NULL), AAFRESULT_NULL_PARAM);
  check(_pHeader->RemoveMob(_pMob));
  checkhr(_pHeader->RemoveMob(_pMob), AAFRESULT_MOB_NOT_FOUND);
  
  // Add it again, for real this time
  check(_pHeader->AddMob(_pMob));
  createEssenceData(_pSourceMob);

  // Cleanup instance data for reuse...
  _pEssenceDescriptor->Release();
  _pEssenceDescriptor = NULL;

  _pFileDescriptor->Release();
  _pFileDescriptor = NULL;

  _pMob->Release();
  _pMob = NULL;

  _pSourceMob->Release();
  _pSourceMob = NULL;
}

void HeaderTest::createEssenceData(IAAFSourceMob *pSourceMob)
{
  assert(_pFile && _pHeader && _pDictionary);
  assert(pSourceMob);
  assert(NULL == _pEssenceData);

  CAAFBuiltinDefs defs (_pDictionary);

  // Attempt to create an AAFEssenceData.
  check(defs.cdEssenceData()->
		CreateInstance(IID_IAAFEssenceData,
					   (IUnknown **)&_pEssenceData));

  check(_pEssenceData->SetFileMob(pSourceMob));
  // AddEssenceData
  checkhr(_pHeader->AddEssenceData(NULL), AAFRESULT_NULL_PARAM);  
  check(_pHeader->AddEssenceData(_pEssenceData));
  checkhr(_pHeader->AddEssenceData(_pEssenceData), AAFRESULT_DUPLICATE_MOBID);  

  // RemoveEssenceData
  checkhr(_pHeader->RemoveEssenceData(NULL), AAFRESULT_NULL_PARAM);  
  check(_pHeader->RemoveEssenceData(_pEssenceData));
  checkhr(_pHeader->RemoveEssenceData(_pEssenceData), AAFRESULT_ESSENCE_NOT_FOUND);  

  // Add it for real this time
  check(_pHeader->AddEssenceData(_pEssenceData));

  _pEssenceData->Release();
  _pEssenceData = NULL;
}

void HeaderTest::openMobs()
{
  aafUInt32	srchItem;
  bool		found;

  assert(_pFile && _pHeader);
  assert(NULL == _pEnumMobs);
  assert(NULL == _pMob);
  assert(NULL == _pSourceMob);


  // CountMobs
  aafNumSlots_t mobCount = 0;
  checkhr(_pHeader->CountMobs(kAAFAllMob, NULL), AAFRESULT_NULL_PARAM);
  checkhr(_pHeader->CountMobs(-1, NULL), AAFRESULT_INVALID_ENUM_VALUE);
  check(_pHeader->CountMobs(kAAFAllMob, &mobCount));
  if (gMaxMobCount != mobCount)
    check(AAFRESULT_TEST_FAILED);
 
  // GetMobs
  checkhr(_pHeader->GetMobs(NULL, NULL), AAFRESULT_NULL_PARAM);
  check(_pHeader->GetMobs(NULL, &_pEnumMobs));
  for (aafUInt32 item = 0; item < mobCount; ++item)
  {
    check(_pEnumMobs->NextOne(&_pMob));

    // Validate the mob data.
	aafMobID_t mobID;
    check(_pMob->GetMobID(&mobID));
	for(srchItem = 0, found = false; !found && srchItem < mobCount; ++srchItem)
	{
		 if (0 == memcmp(&mobID, &_mobID[srchItem], sizeof(mobID)))
		 {
			 found = true;
			 break;
		 }
	}
	if(!found)
	{
		check(AAFRESULT_TEST_FAILED);
	}

	wchar_t mobName[128], expectedMobName[128];
    aafUInt32 expectedMobNameLen = (formatMobName(srchItem, expectedMobName) + 1) * sizeof(wchar_t);
	aafUInt32 mobNameLen;
	check(_pMob->GetNameBufLen(&mobNameLen));
	if (mobNameLen != expectedMobNameLen)
      check(AAFRESULT_TEST_FAILED);
    check(_pMob->GetName(mobName, 128));
    if (0 != memcmp(mobName, expectedMobName, expectedMobNameLen))
      check(AAFRESULT_TEST_FAILED);
	
	// Make sure that we actually created a source mob.
	check(_pMob->QueryInterface(IID_IAAFSourceMob, (void **)&_pSourceMob));
	
    _pSourceMob->Release();
    _pSourceMob = NULL;

    _pMob->Release();
    _pMob = NULL;
  }

  // LookupMob
  static const aafMobID_t  _badMobID = 	{{0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF},
											0xFF, 0xFF, 0xFF, 0xFF,
										{0xFFFFFFFF, 0xFFFF, 0xFFFF, {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF}}};
										

  checkhr(_pHeader->LookupMob(_mobID[0], NULL), AAFRESULT_NULL_PARAM);
  checkhr(_pHeader->LookupMob(_badMobID, &_pMob), AAFRESULT_MOB_NOT_FOUND);
  check(_pHeader->LookupMob(_mobID[0], &_pMob));


  _pEnumMobs->Release();
  _pEnumMobs = NULL;
}

void HeaderTest::openEssenceData()
{
  assert(_pFile && _pHeader);
  assert(NULL == _pEnumEssenceData);
  assert(NULL == _pEssenceData);
  assert(NULL == _pSourceMob);

  // IsEssenceDataPresent
  aafBool 	pResult = false;
  check(_pHeader->IsEssenceDataPresent(_mobID[0], kAAFForeignMedia, &pResult));
  if (pResult == false) check(AAFRESULT_TEST_FAILED);  
  check(_pHeader->IsEssenceDataPresent(_mobID[0], kAAFEssence, &pResult));  
  if (pResult == false) check(AAFRESULT_TEST_FAILED);  


  // CountEssenceData
  checkhr(_pHeader->CountEssenceData(NULL), AAFRESULT_NULL_PARAM);  
  aafUInt32 essenceDataCount = 0;
  check(_pHeader->CountEssenceData(&essenceDataCount));
  if (gMaxMobCount != essenceDataCount)
    check(AAFRESULT_TEST_FAILED);
 
  // LookupEssenceData
  check(_pHeader->LookupEssenceData(_mobID[0], &_pEssenceData));  
  // Make sure that the essence data still references
  // a valid mob.
  check(_pEssenceData->GetFileMob(&_pSourceMob));
  _pSourceMob->Release();
  _pSourceMob = NULL;
  _pEssenceData->Release();
  _pEssenceData = NULL;

  // EnumEssenceData
  checkhr(_pHeader->EnumEssenceData(NULL), AAFRESULT_NULL_PARAM);   
  check(_pHeader->EnumEssenceData(&_pEnumEssenceData));
  for (aafUInt32 item = 0; item < essenceDataCount; ++item)
  {
    check(_pEnumEssenceData->NextOne(&_pEssenceData));

    // Make sure that the essence data still references
    // a valid mob.
    check(_pEssenceData->GetFileMob(&_pSourceMob));

    _pSourceMob->Release();
    _pSourceMob = NULL;

    _pEssenceData->Release();
    _pEssenceData = NULL;
  }


  _pEnumEssenceData->Release();
  _pEnumEssenceData = NULL;
}

void HeaderTest::checkByteOrder()
{
  assert(_pHeader);

  check(_pHeader->QueryInterface(IID_IAAFEndian, reinterpret_cast<void**>(&_pEndian) ) );

  //little endian tests blanked out because they will fail on mac/sparc systems since
  //those systems use big endian. intel byte order

  eAAFByteOrder_t result;

  //test GetStoredByteOrder method
  if(_pEndian->GetStoredByteOrder(&result) != AAFRESULT_SUCCESS)
    check(AAFRESULT_TEST_FAILED);

  //ensure that the returned val is little endian
  if(result != kAAFByteOrderLittle && result != kAAFByteOrderBig)
    check(AAFRESULT_TEST_FAILED);

  //test GetNativeByteOrder method
  if(_pEndian->GetNativeByteOrder(&result) != AAFRESULT_SUCCESS)
    check(AAFRESULT_TEST_FAILED);

  //ensure that the returned val is little endian
  if(result != kAAFByteOrderLittle && result != kAAFByteOrderBig)
    check(AAFRESULT_TEST_FAILED);

  _pEndian->Release();
  _pEndian = NULL;
}

void HeaderTest::checkOperationalPattern()
{
  assert(_pHeader2);

  aafUID_t getOpPatternID;

  //test getOperationalPattern method
  if(_pHeader2->GetOperationalPattern(&getOpPatternID) != AAFRESULT_SUCCESS)
    check(AAFRESULT_TEST_FAILED);

  //ensure the retrieved operational pattern is the same as the set pattern
  if(setOpPatternID != getOpPatternID)   
    check(AAFRESULT_TEST_FAILED);

}

void HeaderTest::checkEssenceContainer()
{
  assert(_pHeader2);

  //Essence containers created in createFileMob() so that below methods could be tested
  //NOTE:_pHeader2 init'd in (this*)checkOperationalPattern()
  if(_pHeader2->UpdateEssenceContainers() != AAFRESULT_SUCCESS)
    check(AAFRESULT_TEST_FAILED); 
  //
  aafUInt32 count;
  if(_pHeader2->CountEssenceContainers(&count) != AAFRESULT_SUCCESS)
    check(AAFRESULT_TEST_FAILED);

  aafUID_t* pContIDs = new aafUID_t [count]; 
  if(_pHeader2->GetEssenceContainers(count, pContIDs) != AAFRESULT_SUCCESS)
    check(AAFRESULT_TEST_FAILED);

  //check essence containers
  for (aafUInt32 c = 0; c < count; ++c)
  {
    aafBoolean_t isPresent = false;
    check(_pHeader2->IsEssenceContainerPresent(pContIDs[c], &isPresent));

    //ensure the container was found
    if(isPresent == false)
      check(AAFRESULT_TEST_FAILED); 

    //ensure the container is used by at least one of the mobs
    bool found = false;
    for (aafUInt32 item = 0; item < gMaxMobCount; ++item)
    {
      aafUID_t containerFormatID = getMobContainerFormatID(item);
      if(containerFormatID == pContIDs[c])
      {
        found = true;
        break;
      }
    }
    if(found != true)
      check(AAFRESULT_TEST_FAILED);
  }

  // For each file mob ensure that the container definition it
  // references is among essence containers in the header.
  for (aafUInt32 item = 0; item < gMaxMobCount; ++item)
  {
    aafUID_t containerFormatID = getMobContainerFormatID(item);
    aafBoolean_t isPresent = false;
    check(_pHeader2->IsEssenceContainerPresent(containerFormatID, &isPresent));
    //ensure the container was found
    if(isPresent == false)
      check(AAFRESULT_TEST_FAILED); 
  }

  delete [] pContIDs;

}

aafUID_t HeaderTest::getMobContainerFormatID(aafUInt32 itemNumber)
{
  assert(_pHeader);
  assert(0 <= itemNumber && gMaxMobCount > itemNumber);

  IAAFMob* pMob = NULL;
  check(_pHeader->LookupMob(_mobID[itemNumber], &pMob));

  IAAFSourceMob* pSourceMob = NULL;
  check(pMob->QueryInterface(IID_IAAFSourceMob, reinterpret_cast<void**>(&pSourceMob)));

  IAAFEssenceDescriptor* pEssenceDescriptor = NULL;
  check(pSourceMob->GetEssenceDescriptor(&pEssenceDescriptor));

  IAAFFileDescriptor* pFileDescriptor = NULL;
  check(pEssenceDescriptor->QueryInterface(IID_IAAFFileDescriptor, reinterpret_cast<void**>(&pFileDescriptor)));

  IAAFContainerDef* pContainerDef = 0;
  check(pFileDescriptor->GetContainerFormat(&pContainerDef));

  IAAFDefObject* pDefObject = NULL;
  check(pContainerDef->QueryInterface(IID_IAAFDefObject, reinterpret_cast<void**>(&pDefObject)));

  aafUID_t  containerDefID;
  check(pDefObject->GetAUID(&containerDefID));

  if (NULL != pMob)
  {
    pMob->Release();
    pMob = NULL;
  }
  if (NULL != pSourceMob)
  {
    pSourceMob->Release();
    pSourceMob = NULL;
  }
  if (NULL != pEssenceDescriptor)
  {
    pEssenceDescriptor->Release();
    pEssenceDescriptor = NULL;
  }
  if (NULL != pFileDescriptor)
  {
    pFileDescriptor->Release();
    pFileDescriptor = NULL;
  }
  if (NULL != pContainerDef)
  {
    pContainerDef->Release();
    pContainerDef = NULL;
  }
  if (NULL != pDefObject)
  {
    pDefObject->Release();
    pDefObject = NULL;
  }

  return containerDefID;
}

void HeaderTest::setPrimaryMob(aafUInt32 itemNumber)
{
  assert(_pHeader && _pDictionary);

  IAAFMob* pMob = NULL;
  check(_pHeader->LookupMob(_mobID[itemNumber], &pMob));

  IAAFHeader2 *pHeader2 = NULL;
  check(_pHeader->QueryInterface(IID_IAAFHeader2, (void **)&pHeader2));
  check(pHeader2->SetPrimaryMob(pMob));

  pMob->Release();
  pMob = NULL;

  pHeader2->Release();
  pHeader2 = NULL;
}

void HeaderTest::readPrimaryMob(aafUInt32 itemNumber)
{
  IAAFHeader2 *pHeader2 = NULL;
  check(_pHeader->QueryInterface(IID_IAAFHeader2, (void **)&pHeader2));
 
  IAAFMob* pPrimaryMob = NULL;
  check(pHeader2->GetPrimaryMob(&pPrimaryMob));

  aafMobID_t primaryMobID = {0};
  check(pPrimaryMob->GetMobID(&primaryMobID));
  if (0 != memcmp(&primaryMobID, &(_mobID[itemNumber]), sizeof(aafMobID_t)))
    check(AAFRESULT_TEST_FAILED);

  pPrimaryMob->Release();
  pPrimaryMob = NULL;

  pHeader2->Release();
  pHeader2 = NULL;
}
